@@ -3,7 +3,7 @@ package JSON::Schema::Modern::Vocabulary::OpenAPI_3_0;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema OpenAPI 3.0 pseudo-vocabulary
 
-our $VERSION = '0.117';
+our $VERSION = '0.118';
 
 use 5.020;
 use utf8;
@@ -136,7 +136,7 @@ JSON::Schema::Modern::Vocabulary::OpenAPI_3_0 - Implementation of the JSON Schem
 
 =head1 VERSION
 
-version 0.117
+version 0.118
 
 =head1 DESCRIPTION
 
@@ -176,15 +176,15 @@ L<https://json-schema.org>
 
 =item *
 
-L<https://www.openapis.org/>
+L<https://www.openapis.org>
 
 =item *
 
-L<https://learn.openapis.org/>
+L<https://learn.openapis.org>
 
 =item *
 
-L<https://spec.openapis.org/oas/3.0>
+L<https://spec.openapis.org/oas/v3.0>
 
 =back
 
